// drweb-clamd API usage example.
// (C) 2003-2016 Doctor Web. All rights reserved.

#ifndef DRWEB_CLAMD_API_COMMON_H
#define DRWEB_CLAMD_API_COMMON_H

#include <sstream>
#include <stdexcept>
#include <string>
#include <vector>

#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>

#define THROW(X)                              \
    do {                                      \
        std::ostringstream _tmp;              \
        _tmp << X;                            \
        throw std::runtime_error(_tmp.str()); \
    } while(0)

struct parsed_args_t
{
    std::string socket_path;
    std::vector<std::string> files;
    std::string command;
};

parsed_args_t parse_args(int argc, const char* argv[], const std::vector<std::string>& supported_cmds);
int connect_to_clamd(const char* socket_path);

struct scoped_fd_t
{
    scoped_fd_t(int fd) : fd(fd) {}
    ~scoped_fd_t() { if(fd >= 0) close(fd); }
    operator int() { return fd; }
    int release() {
        int released = fd;
        fd = -1;
        return released;
    }
    int fd;
};

void write_exact(int fd, const void* buffer, size_t size);
void read_response(int fd);
std::string read_until(int sock, char term);

void send_by_chunks(int clamd, int fd);
void send_fd(int clamd, int fd);

void scan(int clamd, const std::string& file, const std::string& command);
void instream_scan(int clamd, const std::string& path);
void fildes_scan(int clamd, const std::string& path);

#endif // DRWEB_CLAMD_API_COMMON_H
