// drweb-clamd API usage example.
// Scan all files using single ClamD connection.
// (C) 2003-2016 Doctor Web. All rights reserved.

#include <cstring>
#include <iostream>
#include <signal.h>

#include <assert.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "drweb_clamd_api_common.hpp"

int main(int argc, const char* argv[])
{
    try {
        signal(SIGPIPE, SIG_IGN);

        const std::vector<std::string> supported_cmds = { "SCAN", "INSTREAM", "FILDES" };
        parsed_args_t args = parse_args(argc, argv, supported_cmds);
        scoped_fd_t clamd = connect_to_clamd(args.socket_path.c_str());

        const char* start_session_cmd = "nIDSESSION\n";
        write_exact(clamd, start_session_cmd, std::strlen(start_session_cmd));

        for(const auto& file : args.files) {
            if(args.command == "SCAN")
                scan(clamd, file, args.command);
            else if(args.command == "INSTREAM")
                instream_scan(clamd, file);
            else if(args.command == "FILDES")
                fildes_scan(clamd, file);
            else assert(false);
            std::cout << read_until(clamd, '\n');
        }
        const char* end_session_cmd = "nEND\n";
        write_exact(clamd, end_session_cmd, std::strlen(end_session_cmd));
        return 0;
    } catch(const std::exception& ex) {
        std::cerr << ex.what() << std::endl;
        return 1;
    }
}
