// drweb-clamd API usage example.
// (C) 2003-2016 Doctor Web. All rights reserved.

#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "drweb_clamd_c_api_common.h"

const char* supported_cmds[] = { "SCAN", "CONTSCAN", "MULTISCAN", "INSTREAM", "FILDES", "STREAM", NULL };

static void send_file(int clamd, int fd)
{
    char buf[4096];
    while(1) {
        ssize_t size = read(fd, buf, sizeof(buf) / sizeof(*buf));
        if(size == -1) {
            if(errno == EINTR) continue;
            error("ERROR: read: %s", strerror(errno));
        }
        else if(size == 0) break;
        write_exact(clamd, buf, size);
    }
}

static void stream_scan(int clamd, const char* file_path)
{
    const char* cmd = "nSTREAM\n";
    write_exact(clamd, cmd, strlen(cmd));
    char buf[32];
    size_t n = read_until(clamd, '\n', buf, sizeof(buf) / sizeof(*buf) - 1);
    buf[n] = '\0';

    unsigned int port;
    if(sscanf(buf, "PORT %u\n", &port) != 1)
        error("ERROR: STREAM: protocol violation by ClamD");

    int file = open(file_path, O_RDONLY);
    if(file == -1) error("ERROR: open(%s): %s", file_path, strerror(errno));

    int sock = socket(AF_INET, SOCK_STREAM, 0);
    if(sock == -1) error("ERROR: socket: %s", strerror(errno));

    struct sockaddr_in addr = {0};
    addr.sin_family = AF_INET;
    addr.sin_port = htons((in_port_t)port);
    addr.sin_addr.s_addr = htonl((127 << 24) | 1);
    if(connect(sock, (struct sockaddr*)&addr, sizeof(addr)) == -1)
        error("ERROR: connect(127.0.0.1:%d): %s", port, strerror(errno));

    send_file(sock, file);
    shutdown(sock, SHUT_RDWR);
    close(sock);
}

int main(int argc, const char* argv[])
{
    signal(SIGPIPE, SIG_IGN);

    parsed_args_t args = {0};
    parse_args(argc, argv, &args);
    for(int i = 0; args.files[i] != NULL; ++i) {
        int clamd = connect_to_clamd(args.socket);

        if(strcmp(args.cmd, "SCAN") == 0 || strcmp(args.cmd, "CONTSCAN") == 0 || strcmp(args.cmd, "MULTISCAN") == 0)
            scan(clamd, args.files[i], args.cmd);
        else if(strcmp(args.cmd, "INSTREAM") == 0)
            instream_scan(clamd, args.files[i]);
        else if(strcmp(args.cmd, "FILDES") == 0)
            fildes_scan(clamd, args.files[i]);
        else if(strcmp(args.cmd, "STREAM") == 0)
            stream_scan(clamd, args.files[i]);
        else assert(0);

        read_response(clamd);
        shutdown(clamd, SHUT_RDWR);
        close(clamd);
    }
    return 0;
}
